Program Test;

{ Description: Configuration of GPIO pins. }

{ Bits 17..16 : Pin sensing mechanism. }
Const
  GPIO_PIN_CNF_SENSE_Pos = 16; { Position of SENSE field. }
  GPIO_PIN_CNF_SENSE_Msk = $3 shl GPIO_PIN_CNF_SENSE_Pos; { Bit mask of SENSE field. }
  GPIO_PIN_CNF_SENSE_Disabled = $00;  { Disabled. }
  GPIO_PIN_CNF_SENSE_High = $02;  { Wakeup on high level. }
  GPIO_PIN_CNF_SENSE_Low = $03;   { Wakeup on low level. }

{ Bits 10..8 : Drive configuration. }
Const
  GPIO_PIN_CNF_DRIVE_POS = 8;  { POSITION OF DRIVE FIELD. }
  GPIO_PIN_CNF_DRIVE_MSK = $7 SHL GPIO_PIN_CNF_DRIVE_POS;  { BIT MASK OF DRIVE FIELD. }
  GPIO_PIN_CNF_DRIVE_S0S1 = $00;  { STANDARD '0', STANDARD '1'. }
  GPIO_PIN_CNF_DRIVE_H0S1 = $01;  { HIGH '0', STANDARD '1'. }
  GPIO_PIN_CNF_DRIVE_S0H1 = $02;  { STANDARD '0', HIGH '1'. }
  GPIO_PIN_CNF_DRIVE_H0H1 = $03;  { HIGH '0', HIGH '1'. }
  GPIO_PIN_CNF_DRIVE_D0S1 = $04;  { DISCONNECTED '0', STANDARD '1'. }
  GPIO_PIN_CNF_DRIVE_D0H1 = $05;  { DISCONNECTED '0', HIGH '1'. }
  GPIO_PIN_CNF_DRIVE_S0D1 = $06;  { STANDARD '0', DISCONNECTED '1'. }
  GPIO_PIN_CNF_DRIVE_H0D1 = $07;   { HIGH '0', DISCONNECTED '1'. }

{ Bits 3..2 : Pull-up or -down configuration. }
Const
  GPIO_PIN_CNF_PULL_Pos = 2;  { Position of PULL field. }
  GPIO_PIN_CNF_PULL_Msk = $3 shl GPIO_PIN_CNF_PULL_Pos;  { Bit mask of PULL field. }
  GPIO_PIN_CNF_PULL_Disabled = $00;  { No pull. }
  GPIO_PIN_CNF_PULL_Pulldown = $01;  { Pulldown on pin. }
  GPIO_PIN_CNF_PULL_Pullup = $03;  { Pullup on pin. }

{ Bit 1 : Connect or disconnect input path. }
Const
  GPIO_PIN_CNF_INPUT_Pos = 1;  { Position of INPUT field. }
  GPIO_PIN_CNF_INPUT_Msk = $1 shl GPIO_PIN_CNF_INPUT_Pos;  { Bit mask of INPUT field. }
  GPIO_PIN_CNF_INPUT_Connect = 0;  { Connect input pin. }
  GPIO_PIN_CNF_INPUT_Disconnect = 1;  { Disconnect input pin. }

{ Bit 0 : Pin direction. }
Const
  GPIO_PIN_CNF_DIR_Pos = 0;  { Position of DIR field. }
  GPIO_PIN_CNF_DIR_Msk = $1 shl GPIO_PIN_CNF_DIR_Pos;  { Bit mask of DIR field. }
  GPIO_PIN_CNF_DIR_Input = 0;  { Configure pin as an input pin. }
  GPIO_PIN_CNF_DIR_Output = 1;  { Configure pin as an output pin. }

{ General purpose input and output. (GPIO) }
Type GPIOType = Packed Record
    Reserved0: Packed Array[0..320] Of LongWord;
    PortOut: LongWord;  { Write GPIO port (OUT). }
    BitSet: LongWord;  { Set individual bits in GPIO port (OUTSET). }
    BitClear: LongWord; { Clear individual bits in GPIO port (OUTCLR). }
    PortIn: LongWord;  { Read GPIO port (IN). }
    PinDirection: LongWord;  { Direction of GPIO pins (DIR). }
    SetDirection: LongWord;  { DIR set register (DIRSET). }
    ClearDirection: LongWord;  { DIR clear register (DIRCLR). }
    Reserved1: Packed Array[0..119] Of LongWord;
    PinConfig: Packed Array[0..31] Of LongWord;  { Configuration of GPIO pins (PIN_CNF[32]). }
  End;

Const GPIO_BASE = $50000000;

Var
  GPIO: GPIOType Absolute (GPIO_BASE);

Const { For nRF51DK board LEDs. }
  LED1 = 21;
  LED2 = 22;
  LED3 = 23;
  LED4 = 24;

Var
  LEDsList: Array [1..4] Of LongWord = (LED1, LED2, LED3, LED4);

Var
  Index: Integer;

Procedure GPIOSetPinRangeOutput(PinRangeStart, PinRangeEnd: LongWord); Inline;
Var
  PinIndex: LongWord;
  OutputSetting: LongWord;
Begin
  OutputSetting := (GPIO_PIN_CNF_SENSE_Disabled ShL GPIO_PIN_CNF_SENSE_Pos) Or
                   (GPIO_PIN_CNF_DRIVE_S0S1 ShL GPIO_PIN_CNF_DRIVE_Pos) Or
                   (GPIO_PIN_CNF_PULL_Disabled ShL GPIO_PIN_CNF_PULL_Pos) Or
                   (GPIO_PIN_CNF_INPUT_Disconnect ShL GPIO_PIN_CNF_INPUT_Pos) Or
                   (GPIO_PIN_CNF_DIR_Output ShL GPIO_PIN_CNF_DIR_Pos);
  For PinIndex := PinRangeStart To PinRangeEnd Do
    GPIO.PinConfig[PinIndex] := OutputSetting;
End;

Procedure GPIOTogglePin(PinNumber: LongWord); Inline;
Var
  PinBit: LongWord;
Begin
  PinBit := LongWord(1) ShL PinNumber;
  If (GPIO.PortOut And PinBit)=0 Then
    GPIO.BitSet := PinBit
  Else
    GPIO.BitClear := PinBit;
End;

Procedure MicrosecondsDelay(Count: LongWord); Inline; Assembler; NoStackFrame;
Asm
.Loop:
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  sub r0,r0,#1
  cmp r0, #0
  bne .Loop
End;

Procedure MillisecondsDelay(Count: LongWord); Inline;
Begin
  While Count>0 Do
    Begin
      MicrosecondsDelay(999);
      Dec(Count);
    End;
End;

Begin
  { Configure LED-pins as outputs. }
  GPIOSetPinRangeOutput(LED1, LED4);
  While True Do
    For Index := 1 To 4 Do
      Begin
        GPIOTogglePin(LEDsList[Index]);
        MillisecondsDelay(1000);
      End;
End.

